#!/bin/bash

shopt -s nullglob

DESIRED_GHC=8.2.1

LOGFILE=$0.log

function uhoh {
    echo
    echo "Something went wrong :-("
    echo "--------------------"
    echo "Please show this log to your local techie:"
    echo ""
    echo "$LOGFILE"
}

echo "GenI install log" > $LOGFILE
{
    date
    md5sum $0
    lsb_release -a
    echo
    echo "PATH: $PATH"
} >> $LOGFILE

sleep_until_exists () {
     echo >&2 "Waiting until we've finished downloading $1..."
     while [ ! -e "$1" ]; do
         sleep 1
     done
}

fetch_bg () {
    LOGDIR=$1
    URL=$2
    DEST=$3
    (wget "$URL" -o "$LOGDIR/fetch-$DEST" -O "$DEST.download" && mv "$DEST.download" "$DEST") &
}

ghc --version
if [ $? -ne 127 ]; then
    echo >&2 "Hmm, GHC already exists on your system. Cowardly exiting, sorry."
    echo >&2 "If it's out of date (we want $DESIRED_GHC), you'll want to remove it"
    exit 1
fi

geni --version
if [ $? -ne 127 ]; then
    echo >&2 "Hmm, geni already exists on your system. Cowardly exiting, sorry."
    exit 1
fi

wx-config --version
if [ $? -ne 127 ]; then
    echo >&2 "Hmm, wxWidgets already exists on your system. Cowardly exiting, sorry."
    exit 1
fi

wxdirect --version
if [ $? -ne 127 ]; then
    echo >&2 "Hmm, wxdirect already exists on your system. Cowardly exiting, sorry."
    exit 1
fi

trap uhoh EXIT

# Neat sudo keep-alive trick from
# https://gist.github.com/cowboy/3118588
# Might as well ask for password up-front, right?
sudo -v
# Keep-alive: update existing sudo time stamp if set, otherwise do nothing.
while true; do sudo -n true; sleep 60; kill -0 "$$" || exit; done 2>/dev/null &

set -e

{
echo >&2 '**************************************************'
echo >&2 'Installing GHC and friends'
echo >&2 '**************************************************'

# GenI 0.25 and up requires GHC 8
sudo add-apt-repository --yes ppa:hvr/ghc
sudo apt-get update
sudo apt-get -y --force-yes install ghc-${DESIRED_GHC} cabal-install-2.0\
	libssl-dev\
	graphviz\
	git\
	libwxgtk3.0-dev libwxgtk-media3.0-dev libwxgtk-webview3.0-dev

export PATH=$PATH:/opt/ghc/bin:$HOME/.cabal/bin
cabal update

echo >&2 '**************************************************'
echo >&2 'Installing wxHaskell'
echo >&2 '**************************************************'

# 2017-10-03: wxHaskell 0.92 still depends on Cabal 1.24
# which in turn depends on process-1.4, which seems to be
# problematic with GHC 8.2.1. This branch of the code
# slightly amends things to support Cabal 2.0
git clone https://github.com/kowey/wxHaskell.git
pushd wxHaskell
git checkout ghc-8.2.1
cabal install ./wxdirect ./wxc ./wxcore ./wx
popd

echo >&2 '**************************************************'
echo >&2 'Installing GenI'
echo >&2 '**************************************************'

cabal install GenI geniserver geni-util geni-gui

echo >&2 '*****************************************************************************'
echo >&2 'Success! (hopefully)'
echo >&2 ''
echo >&2 'Now make sure the following is in your .bashrc (or was it .bash_profile?)'
echo >&2 ''
echo >&2 'export PATH=${PATH}:/opt/ghc/bin:${HOME}/.cabal/bin'
echo >&2 '*****************************************************************************'
} 2>&1 | tee -a $LOGFILE

trap '' EXIT
